# bayz.read.fast: new function to read bayz output pretty fast, uses awk to make a temporary file
# (probably only works on unix/linux/mac).
# NOTE: it should be possible to find out the operation system and use this function automatically,
# .Platform$OS.type reports "unix" on both linux and mac and should be useful for this.
bayz.read.fast <- function(basename,parameter,makemcmc=FALSE){
   paraminfo <- bayz.param(basename) # we need to know the size of the parameter to correctly format the output
   parnumber <- match(parameter,paraminfo$Parameter)
   if (is.na(parnumber)) {
   	  print(paste("Parameter",parameter,"is not available in this analysis"))
   	  return(NULL)
   }
   Nlevels <- paraminfo$Levels[parnumber]
   filename1 <- paste(basename,".out",sep="")   # file name alternatives
   filename2 <- paste(basename,".out.gz",sep="")
   filename3 <- paste(basename,".out.xz",sep="")
   fileavail <- file.exists(filename1,filename2,filename3)
   if (sum(fileavail)==0) {
      cat("Bayz sample output file (.out, .out.gz or .out.xz) not available here\n")
      return(NULL)
   }
   # when available, first choice read from out.gz, second choice out.xz and else .out
   if (fileavail[2]) {
      awkstring <- paste("gunzip -c -d ",basename,".out.gz | awk \"(\\$2==\\\"",
                   parameter, "\\\"){\\$2=\\\"\\\"; print }\" "," >temp.out", sep="")
   }
   else {
      if (fileavail[3]) {
         awkstring <- paste("xz -c -d ",basename,".out.xz | awk \"(\\$2==\\\"",
	           parameter, "\\\"){\\$2=\\\"\\\"; print }\" "," >temp.out", sep="")
      }
      else {
          awkstring <- paste("awk \"(\\$2==\\\"", parameter, "\\\"){\\$2=\\\"\\\"; 
                             print }\" ", basename, ".out >temp.out", sep="")
      }
   }
   system(awkstring)
   samples <- scan(file="temp.out",quiet=TRUE)
   if(length(samples)==0) {
   	  print(paste("No samples for",parameter,"in bayz out file"))
   	  return(NULL)
   }
   Ncycles <- length(samples)/(Nlevels+1)
   samples <- t(matrix(samples,(Nlevels+1),Ncycles))
   cycles  <- samples[,1] # cycle numbers are in column 1
   rownames(samples) <- cycles
   samples <- samples[,-1] # remove cycle numbers (they are now the row names)
   if ( class(samples) != "matrix") { # it is possible that R has changed it from matrix
                                      # to numeric when only one column is left
   	  samples <- as.matrix(samples)
   }
   colnames(samples) <- bayz.labels(basename,parameter)
   if (makemcmc) {
     return(bayz2mcmc(samples))
   }
   else {
     return(samples)
   }
}

# bayz 'slow' reading function that should work anywhere, i.e. also in Windows.
# Does not make a temp.out file.
bayz.read <- function(basename,parameter,makemcmc=FALSE){
   paraminfo <- bayz.param(basename) # get parameter info: levels are needed
   parnumber <- match(parameter,paraminfo$Parameter)
   if (is.na(parnumber)) {
   	  print(paste("Parameter",parameter,"is not available in this analysis"))
   	  return(NULL)
   }
   Nlevels <- paraminfo$Levels[parnumber]
   infile <- file(paste(basename,".out",sep=""),"r")
   nlines <- 0
   outdat <- NULL
   cycles <- NULL
   repeat {
      twofields <- scan(infile,what=character(),nmax=2,quiet=TRUE)
      if(length(twofields) ==0) { break }
      if(twofields[2]==parameter) {
      	 outdat <- rbind(outdat,scan(infile,nmax=Nlevels,quiet=TRUE))
      	 cycles <- c(cycles,as.integer(twofields[1]))
         nlines <- nlines+1
      }
      else {
         scan(infile,nlines=1,quiet=TRUE)
      }
   }
   close(infile)
   if(length(outdat)==0) {
   	  print(paste("No samples for",parameter,"in bayz out file"))
   	  return(NULL)
   }
   if ( class(outdat) != "matrix") { # it is possible that R has changed it from matrix to numeric when there is only one column
   	  samples <- as.matrix(samples)
   }
   rownames(outdat) <- cycles
   colnames(outdat) <- bayz.labels(basename,parameter)
   if (makemcmc) {
     return(bayz2mcmc(outdat))
   }
   else {
     return(outdat)
   }
}

# obsolete reading function
read.bayzout <- function (basename,parameter){
   print("Function read.bayzout is removed from bayzRfunc; use read.bayz or read.bayz.fast instead.")
   return()
}

# collect path variables from bayz job file; collects everything that starts with $path...
bayz.path <- function(basename) {
   joblines <- readLines(paste(basename,".bayz",sep=""),n=-1)
   splittext <- strsplit(joblines[nchar(joblines)>0],"=")
   pathvars <- NULL
   paths <- NULL
   for (i in 1:length(splittext)) {
      if(substr(splittext[[i]][1],1,5)=="$path") {
         pathvars <- c(pathvars,substr(splittext[[i]][1],2,nchar(splittext[[i]][1])))
         paths <- c(paths,splittext[[i]][2])
      }
   }
   if (length(paths) >0 ) {
     paths <- as.data.frame(paths)
     rownames(paths) <- pathvars
   }
   return (paths)
}


# bayz.param: shows the parameters and their levels
# (obtained from the lab(els) file). Negative indicated levels are changed to positive.
bayz.param <- function (basename){
   filename1 <- paste(basename,".lab",sep="")   # file name alternatives
   filename2 <- paste(basename,".lab.gz",sep="")
   filename3 <- paste(basename,".lab.xz",sep="")
   fileavail <- file.exists(filename1,filename2,filename3)
   if (sum(fileavail)==0) {
      cat("Bayz label file (.lab, .lab.gz or .lab.xz) not available here\n")
      return(NULL)
   }
   if (fileavail[2]) {
      filename <- filename2
   }
   else {
      if (fileavail[3]) {
         filename <- filename3
      }
      else {
         filename <- filename1
      }
   }
   param <- as.data.frame(scan(file=filename,what=list(character(),integer()),flush=T,quiet=T))
   colnames(param) <- c("Parameter","Levels")
   param$Levels <- abs(param$Levels)
   return(param)
}

# New function to read labels;
# Also accommodates for the parameters with an indicated negative size.
bayz.labels <- function(basename,parameter){
   filename1 <- paste(basename,".lab",sep="")   # file name alternatives
   filename2 <- paste(basename,".lab.gz",sep="")
   filename3 <- paste(basename,".lab.xz",sep="")
   fileavail <- file.exists(filename1,filename2,filename3)
   if (sum(fileavail)==0) {
      cat("Bayz label file (.lab, .lab.gz or .lab.xz) not available here\n")
      return(NULL)
   }
   if (fileavail[2]) {
      filename <- filename2
   }
   else {
      if (fileavail[3]) {
         filename <- filename3
      }
      else {
         filename <- filename1
      }
   }
   # get first 3 fields from label file to check availability and determine number of levels
   param <- scan(file=filename,what=list(character(),integer(),character()),flush=T,quiet=T)
   parrow <- match(parameter,param[[1]])
   if (is.na(parrow)) {
   	  stop("Parameter is not available in this analysis")
   }
   Nlevels <- param[[2]][parrow]
   while (Nlevels < 0 ){
   	  parrow <- match(param[[3]][parrow],param[[1]])
   	  Nlevels <- param[[2]][parrow]
   }
   labels <- scan(file=filename,what=character(),skip=(parrow-1),nmax=(Nlevels+2),quiet=T)
   return(labels[c(-1,-2)])
}

# obsolete label retrieval function
read.bayzlab <- function (basename,parameter){
    print("Function read.bayzlab is removed from bayzRfunc; use bayz.labels instead.")
    return()
}

# bayz2mcmc: convert from bayz output to mcmc class.
# This is now updated to accommodate for the new reading functions, where the cycle numbers
# have been put in the rownames.
# March 2015: mcmc() goes wrong when size of samples is not (last-first)/skip + 1; this can happen
# when bayz output is partly missing (very rare cases, e.g. disk full while running) - now that's repaired but probably not perfect.
bayz2mcmc <- function(out) {
	nrows <- dim(out)[1]
	firstcycle = as.integer(rownames(out)[1])
	secondcycle = as.integer(rownames(out)[2])
   skip = secondcycle - firstcycle
   lastcycle = firstcycle + (nrows-1)*skip
   if ( lastcycle != as.integer(rownames(out)[nrows]) ) {
      cat(paste("NOTE: Cycle numbers in bayz output appear incorrect and were reset to ",firstcycle,"to",lastcycle,"with skip",skip))
   }
	return(mcmc(out,start=firstcycle,end=lastcycle,thin=skip))
}

# poststat: compute the simple posterior statistics mean and stdev
# using table obtained with read.bayzout
poststat <- function(out) {
	res <- rbind(apply(out,2,mean),apply(out,2,sd))
	row.names(res) <- c("post.mean","post.stdev")
	t(res)
}

summary.bayz <- function(out) {
	res <- rbind(apply(out,2,mean),apply(out,2,sd))
	row.names(res) <- c("post.mean","post.stdev")
	t(res)
}

eiginput <- function(R) {
	R <- as.matrix(R)
	nrows <- dim(R)[1]
	ncols <- dim(R)[2]
	if( nrows != ncols ) {
		print("Input not a square matrix")
		return()
		}
	if ( ! isSymmetric(R)) {
		print("Input matrix is not symmetric")
		return()
		}
	eigR <- eigen(R)
	out <- NULL
	for (i in 1:nrows) {
		if (eigR$vectors[nrows,i] < 0) {
			out <- cbind(out, -eigR$vectors[1:nrows-1,i])
		}
		else {
			out <- cbind(out, eigR$vectors[1:nrows-1,i])
			}
		out <- cbind(out,eigR$values[i])
	}
	return(out)
}

# --- hpdpos: computation of HPD interval for positive value.
# When pameters have a boundary, the HPD is not correctly computed using the standard density and
# HPDinterval (from coda) functions, because the densities can be smoothed 'over' the boundary.
# Usage: hpdpos(values,prob=...). prob= sets the HPD interval to compute, e.g. 0.95 for 0.95 HPD interval.
# The default is 0.95. The density computation uses standard R density() function with gaussian kernel,
# which *probably* can also be modified by passing kernel= (see density function help).
# --- Demonstration
# x <- -log(runif(1000)) # Exponentially distributed variates
# plot(density(x))    # plot of density will incorrectly show density that is 'smoothed down' and continued below 0
# library(coda)
# HPDinterval(as.mcmc(x),prob=0.95)  # also this is wrong, because HPD should start at 0
#             lower    upper
# var1 0.0002623947 2.860773
# correct computation using 'hpdpos' function shows left bound at zero
# hpdpos(x)
# [1] "HPD Left: 0.000000 Right: 2.766189"

hpdpos <- function(x, prob=0.95) {
   if( prob < 0 || prob > 1) {
      return("ERROR: prob not from 0-1")
   }
   origbw <- density(x)$bw
   xreflected <- c(-x,x)
   d <- density(xreflected,bw=origbw)
   n <- length(d$x)
   d$x <- d$x[(n/2):n]
   d$y <- d$y[(n/2):n]
   if(d$x[1] < 0 ) d$x[1] <- 0
#   binwidth <- d$x[3]-d$x[2]
   d$y <- d$y/sum(d$y)
   n <- length(d$x)
   hpdleft <- 1
   hpdright <- n
   tailprob <- 0
   while ( (tailprob < (1-prob)) ) {
      if(d$y[hpdleft] < d$y[hpdright]) {
         tailprob <- tailprob + d$y[hpdleft]
         hpdleft <- hpdleft + 1
      }
      else {
         tailprob <- tailprob + d$y[hpdright]
         hpdright <- hpdright -1
      }
   }
   result <- c(d$x[hpdleft],d$x[hpdright])
   names(result) <- c("HPD-left","HPD-right")
   return(result)
}

denspos <- function(x) {

   origbw <- density(x)$bw
   xreflected <- c(-x,x)
   d <- density(xreflected,bw=origbw)
   n <- length(d$x)
   d$x <- d$x[(n/2):n]
   d$y <- d$y[(n/2):n]
   if(d$x[1] < 0 ) d$x[1] <- 0
   return(d)

}

# DIC version1: for single trait models (or SEM models…).
# This one retrieves from output resid.trait and var.resid.trait.
dic1 <- function(basename,traitname,readfast=FALSE) {
  out <- NULL
  for(i in 1:length(traitname)) {
	if(readfast) {
		resid <- bayz.read.fast(basename,paste("resid.",traitname[i],sep=""))
		vare <- bayz.read.fast(basename,paste("var.resid.",traitname[i],sep=""))
		if(length(resid)==0 | length(vare)==0) {
			return(NULL)
		}
	}
	else {
		resid <- bayz.read(basename,paste("resid.",traitname[i],sep=""))
		vare <- bayz.read(basename,paste("var.resid.",traitname[i],sep=""))
		if(length(resid)==0 | length(vare)==0) {
			return(NULL)
		}
	}
	combsamp <- cbind(resid,vare) # combined (residual+variance) samples
	N <- dim(combsamp)[2] # size of residuals + 1 variance
	dic_samples <- apply(combsamp,1,function(x){(N-1)*log(x[N])+sum(x[1:(N-1)]^2)/x[N]})
	vare.pm <- mean(vare)
	resid.pm <- colMeans(resid)
	dicmean <- (N-1)*log(vare.pm)+sum(resid.pm^2)/vare.pm
	out <- rbind(out,c((N-1),(2*mean(dic_samples) - dicmean),(mean(dic_samples)-dicmean)))
  }
  if (length(traitname)>1) {
    totals <- colSums(out)
    out <- rbind(out,totals)
    rownames(out) <- c(traitname,"Total")
  }
  else {
    rownames(out) <- traitname
  }
  colnames(out) <- c("Nresid","DIC","pD")
  cat("*** warning *** Consult the manual when/where the DIC(s) computed here is/are correct\n")
  return(out)
}

# DIC version2: for multi trait models using lvec.
# This one retrieves from output resid.trait (for all traits) and noise.var.resid.*.
dic2 <- function(basename,readfast=FALSE) {
  out <- NULL
  traitname <- bayz.labels(basename,"noise.var.resid.*")
  if(length(traitname)==0) {
	return(NULL)
  }
  # the noise variance needed can be read in one step
  if (readfast) {
    vare <- bayz.read.fast(basename,"noise.var.resid.*")
  }
  else {
    vare <- bayz.read(basename,"noise.var.resid.*")
  }
  # the residuals needed are read per trait, and DIC are computed and collected
  for(i in 1:length(traitname)) {
	if(readfast) {
		resid <- bayz.read.fast(basename,paste("resid.",traitname[i],sep=""))
		if(length(resid)==0) {
			return(NULL)
		}
	}
	else {
		resid <- bayz.read(basename,paste("resid.",traitname[i],sep=""))
		if(length(resid)==0) {
			return(NULL)
		}
	}
	combsamp <- cbind(resid,vare[,i]) # combined (residual+variance) samples
	N <- dim(combsamp)[2] # size of residuals + 1 variance
	dic_samples <- apply(combsamp,1,function(x){(N-1)*log(x[N])+sum(x[1:(N-1)]^2)/x[N]})
	vare.pm <- mean(vare[,i])
	resid.pm <- colMeans(resid)
	dicmean <- (N-1)*log(vare.pm)+sum(resid.pm^2)/vare.pm
	out <- rbind(out,c((N-1),(2*mean(dic_samples) - dicmean),(mean(dic_samples)-dicmean)))
  }
  totals <- colSums(out)
  out <- rbind(out,totals)
  rownames(out) <- c(traitname,"Total")
  colnames(out) <- c("Nresid","DIC","pD")
  cat("*** warning *** Consult the manual when/where the DIC(s) computed here is/are correct\n")
  return(out)
}
